//THIS CODE ADAPTED FROM
/*
VarintBitConverter: https://github.com/topas/VarintBitConverter 
Copyright (c) 2011 Tomas Pastorek, Ixone.cz. All rights reserved.
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above
    copyright notice, this list of conditions and the following
    disclaimer in the documentation and/or other materials provided
    with the distribution.
THIS SOFTWARE IS PROVIDED BY TOMAS PASTOREK AND CONTRIBUTORS ``AS IS'' 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TOMAS PASTOREK OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE. 
*/

class VarintTranslator {
  static const int allButMsb = 0x7f;
  static const int justMsb = 0x80;

  static int popVarint(List<int> bytes) {
    int result = 0;
    var currentShift = 0;
    var bytesPopped = 0;

    for (int i = 0; i < bytes.length; i++) {
      bytesPopped++;
      var current = bytes[i] & allButMsb;
      result |= current << currentShift;

      if ((bytes[i] & justMsb) != justMsb) {
        bytes.removeRange(0, bytesPopped);
        return result;
      }

      currentShift += 7;
    }

    throw ArgumentError("Byte array did not contain valid varints.");
  }

  static List<int> getVarint(int value) {
    var buff = List<int>.generate(10, (idx) => 0);
    var currentIndex = 0;

    if (value == 0) {
      return [0];
    }

    while (value != 0) {
      var byteVal = value & allButMsb;
      value >>= 7;

      if (value != 0) {
        byteVal |= 0x80;
      }

      buff[currentIndex++] = byteVal;
    }

    List<int> result = List.generate(currentIndex, (idx) => 0);
    for (var i = 0; i < currentIndex; i++) {
      result[i] = buff[i];
    }

    return result;
  }
}
